﻿/**
* @file StovePCDefine.h
* @brief StovePCDefine 헤더파일
* @details StovePCSDK 사용하는 enum, struct를 선언합니다.
*/
#ifndef STOVEPCDEFINE_H_
#define STOVEPCDEFINE_H_

#include "StovePCEnum.h"

 #ifndef int64_t
 typedef long long int64_t;
 #endif
 #ifndef uint64_t
 typedef unsigned long long uint64_t;
 #endif

/*!
*  @struct StovePCError
*  @brief OnError Callback 호출시 전달받는 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCError
{
    /// 호출된 함수를 나타내는 enum 값
    StovePCFunctionType functionType;

    /// Error 에 대한 유형 결과값 
    StovePCResult result;

    /// Error가 발생한 이유에 대한 상세 메세지
    char* message;

    /// 외부에서 발생한 Error 값 (HTTP error or API result code)
    int externalError;
};
#pragma pack(pop)

/*!
*  @struct StovePCConfig
*  @brief SDK Int 호출시 설정되는 환경 값입니다.
*/
struct StovePCConfig
{
    /// StovePCSDK의  Stove 환경: "QA", "SANDBOX", "LIVE"
    char* env;

    /// 스토브에서 발급 받은 Application Key
    char* appKey;

    /// 스토브에서 발급 받은 Secret Key
    char* secretKey;

    /// 스토브에서 사용할 game ID
    wchar_t* gameId;

    /// SDK 로그 레벨 설정 
    StovePCLogLevel logLevel;

    /// SDK 로그경로 설정 (empty string이면 현재 실행 경로에 생성 )
    wchar_t* logPath;
};

/*!
*  @struct StovePCToken
*  @brief StovePC_GetToken API 호출시 응답받는 콜백의 Token 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCToken
{
    /// 런처에서 발행한 토큰 값
    char* accessToken;
};
#pragma pack(pop)

/*!
*  @struct StovePCUser
*  @brief StovePC_GetUser API 호출시 응답받는 콜백의 UserInfo 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCUser
{
    /// 런처에서 로그인한 유저의 스토브 회원번호 
    uint64_t memberNo;

    /// 런처에서 로그인한 유저의 스토브 닉네임 
    wchar_t* nickname;

    /// 런처에서 로그인한 유저의 gameUserId(GameID + MemeberNo)
    char* gameUserId; 
};
#pragma pack(pop)

/*!
*  @struct StovePCOwnership
*  @brief StovePC_GetOwnership 관련 API 호출시 응답받는 콜백의 OwnerShip 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCOwnership
{
    /// Ownership 게임 아이디 
    wchar_t* gameId;

    /// Ownership 게임 코드 
    int gameCode;

    /// Ownership 코드 
    int ownershipCode;

    /// Ownership 구매날짜 
    uint64_t purchaseDate;

    /// Ownership 멤버넘버 
    uint64_t memberNo;
};
#pragma pack(pop)

/*!
*  @struct StovePCStatFullId
*  @brief StovePC_GetStat 관련 API 호출시 응답받는 콜백의 StatFullId 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCStatFullId
{
    /// Stats 게임 아이디 
    wchar_t* gameId;

    /// Stat 아이디 [A-Z0-9\\-_]*$ 형식(영문 대문자, 숫자, underscore, hyphen)
    char* statId;
};
#pragma pack(pop)

/*!
*  @struct StovePCStat
*  @brief StovePC_GetStat 관련 API 호출시 응답받는 콜백의 Stat 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCStat
{
    /// Stat 전체아이디
    StovePCStatFullId statFullId;

    /// Stat 현재값
    int currentValue;

    /// 멤버넘버 
    uint64_t memberNo;

    /// Stat 최종 업데이트 시간(epoch time milliseconds)
    uint64_t updatedAt;
};
#pragma pack(pop)

/*!
*  @struct StovePCStatValue
*  @brief StovePC_SetStat 관련 API 호출시 응답받는 콜백의 StatValue 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCStatValue
{
    /// Stat 현재값
    int currentValue;

	/// Stat 값의 변경 여부
	bool updated;

    /// 오류 메시지 (eg. integer overflow) 
    char* errorMessage;
};
#pragma pack(pop)

/*!
*  @struct StovePCAchievementCondition
*  @brief StovePC_GetAchievement, StovePC_GetAllAchievement 관련 API 호출시 응답받는 콜백의 AchievementCondition 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCAchievementCondition
{
    /// Achievement 달성 목표값
    int goalValue;

    /// 값 처리 방법 (DELTA_SUM: 누적 데이터 합계, REPLACE: 기존 데이터 교체, MAX : 큰값으로 대체, MIN : 작은값으로 대체)
    char* valueOperation;

    /// Achievement 목표값과 사용자 업적 값의 비교 연산 방식 ( GTE: >=, LTE: <= )
    char* type;
};
#pragma pack(pop)

/*!
*  @struct StovePCAchievement
*  @brief StovePC_GetAchievement, StovePC_GetAllAchievement 관련 API 호출시 응답받는 콜백의 Achievement 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCAchievement
{
    /// Achievement 아이디
    char* achievementId;

    /// Achievement 이름
    wchar_t* name;

    /// Achievement 설명
    wchar_t* description;

    /// 기본 이미지 URL
    wchar_t* defaultImage;

    /// 달성 이미지 URL
    wchar_t* achievedImage;

    /// Achievement 조건
    StovePCAchievementCondition condition;

    /// Achievement 현재값
    int value;

    /// Achievement 상태 (ONGOING : 진행중, ACHIEVED: 업적 달성)
    char* status;
};
#pragma pack(pop)

/*!
*  @struct StovePCRank
*  @brief StovePC_GetRank 관련 API 호출시 응답받는 콜백의 Rank 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCRank
{
    /// 멤버넘버 
    uint64_t memberNo;

    /// 점수
    int score;

    /// 순위
    unsigned int rank;

    /// 닉네임
    wchar_t* nickname;

    /// 프로필 이미지 URL
    wchar_t* profileImage;
};
#pragma pack(pop)

/*!
*  @struct StovePCPopupControlUIVisible
*  @brief Popup 관련 API 호출시 응답받는 콜백의 PopupControlUIVisible 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCPopupControlUIVisible
{
    /// 닫기 버튼 표시 여부
    bool closeButton;

    /// 탐색 막대 표시 여부
    bool navigationBar;

    /// 뒤로 버튼 표시 여부
    bool backButton;

    /// 앞으로 버튼 표시 여부
    bool forwardButton;

    /// 새로고침 버튼 표시 여부
    bool refreshButton;

    /// 홈 버튼 표시 여부
    bool homeButton;

    /// 그만 보기 버튼 표시 여부
    bool disallowedButton;
};
#pragma pack(pop)

/*!
*  @struct StovePCPopupControlUICloseButtonImageDetail
*  @brief Popup 관련 API 호출시 응답받는 콜백의 PopupControlUICloseButtonImageDetail 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCPopupControlUICloseButtonImageDetail
{
    char* fileUrl;

    char* fileId;
};
#pragma pack(pop)

/*!
*  @struct StovePCPopupControlUICloseButtonImage
*  @brief Popup 관련 API 호출시 응답받는 콜백의 PopupControlUICloseButtonImage 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCPopupControlUICloseButtonImage
{
    /// 보통 상태 버튼 정보
    StovePCPopupControlUICloseButtonImageDetail normal;

    /// 눌린 상태 버튼 정보
    StovePCPopupControlUICloseButtonImageDetail pressed;

    /// 닫기 버튼 유형 (커스텀 : 0, 기본값 1, 프리셋: 1 ~ 6)
    int type;
};
#pragma pack(pop)

/*!
*  @struct StovePCPopupControlUI
*  @brief Popup 관련 API 호출시 응답받는 콜백의 PopupControlUI 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCPopupControlUI
{
    /// 각 버튼의 표시 여부
    StovePCPopupControlUIVisible visible;

    /// 그만 보기 날짜 범위
    int disallowedDay;

    /// 닫기 버튼 커스터마이징
    StovePCPopupControlUICloseButtonImage closeButtonImage;
};
#pragma pack(pop)

/*!
*  @struct StovePCPopupControl
*  @brief 팝업 관련 API 호출시 응답받는 콜백의 PopupControl 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCPopupControl
{
    /// 사용자 인터페이스
    StovePCPopupControlUI ui;
};
#pragma pack(pop)

/*!
*  @struct StovePCAutoPopup
*  @brief StovePC_GetAutoPopup 관련 API 호출시 응답받는 콜백의 AutoPopup 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCAutoPopup
{
    /// POPUP - 일반팝업, CROSSPROMOTION - 크로스프로모션팝업
    char* origin;

    /// 고유 번호
    int id;

    /// 웹뷰 생성 요청 URL
    char* url;

    /// 팝업 컨트롤 정보
    StovePCPopupControl control;
};
#pragma pack(pop)

/*!
*  @struct StovePCManualPopup
*  @brief StovePC_GetManualPopup 관련 API 호출시 응답받는 콜백의 ManualPopup 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCManualPopup
{
    /// POPUP - 일반팝업, CROSSPROMOTION - 크로스프로모션팝업
    char* origin;

    /// 수동팝업 노출 트리거 키
    char* resourceKey;
    
    /// 고유 번호
    int id;

    /// 웹뷰 생성 요청 URL
    char* url;

    /// 팝업 컨트롤 정보
    StovePCPopupControl control;
};
#pragma pack(pop)

/*!
*  @struct StovePCNewsPopup
*  @brief StovePC_GetNewsPopup 관련 API 호출시 응답받는 콜백의 NewsPopup 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCNewsPopup
{
    /// POPUP - 일반팝업, CROSSPROMOTION - 크로스프로모션팝업
    char* origin;

    /// 고유 번호
    int id;

    /// 웹뷰 생성 요청 URL
    char* url;

    /// 팝업 컨트롤 정보
    StovePCPopupControl control;
};
#pragma pack(pop)

/*!
*  @struct StovePCCouponPopup
*  @brief StovePC_GetCouponPopup 관련 API 호출시 응답받는 콜백의 CouponPopup 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCCouponPopup
{
    /// 웹뷰 생성 요청 URL
    char* url;

    /// 팝업 컨트롤 정보
    StovePCPopupControl control;
};
#pragma pack(pop)

/*!
*  @struct StovePCCommunityPopup
*  @brief StovePC_GetCommunityPopup 관련 API 호출시 응답받는 콜백의 CommunityPopup 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCCommunityPopup
{
    /// 웹뷰 생성 요청 URL
    char* url;

    /// 팝업 컨트롤 정보
    StovePCPopupControl control;
};
#pragma pack(pop)


/*!
*  @struct StovePCPopupRequestHeader
*  @brief Popup 관련 API 호출시 응답받는 콜백의 PopupRequestHeader 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCPopupRequestHeader
{
    /// 이름
    char* name;

    /// 값
    char* value;
};
#pragma pack(pop)

/*!
*  @struct StovePCPopupRequestCookie
*  @brief Popup 관련 API 호출시 응답받는 콜백의 PopupRequestHCookie 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCPopupRequestCookie
{
    /// 이름
    char* name;

    /// 값
    char* value;
};
#pragma pack(pop)

/*!
*  @struct StovePCCustomEvent
*  @brief CustomLog 관련 API 호출시 응답받는 콜백의 StovePCCustomEvent 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCCustomEvent
{
    /// 이벤트 이름
    char* name;

    /// 1차 카테고리
    char* category1;

    /// 2차 카테고리
    char* category2;

    /// 값
    float simpleValue;
};
#pragma pack(pop)

/*!
*  @struct StovePCCustomEventParameter
*  @brief CustomLog 관련 API 호출시 응답받는 콜백의 StovePCCustomEventParameter 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCCustomEventParameter
{
    /// 이름
    char* name;

    /// 값
    char* value;
};
#pragma pack(pop)

/*!
*  @struct StovePCOVerImmersion
*  @brief StovePCOverImmersion 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCOverImmersion
{
    /// 과몰입 경고 메세지
    wchar_t* message;

    /// 게임 경과시간(시간)
    int elapsedTimeInHours;

    /// 메세지 최소 노출시간(초)
    int minExposureTimeInSeconds;
};
#pragma pack(pop)

/*!
*  @struct StovePCShutdown
*  @brief StovePCShutdown 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCShutdown
{
    /// 사용자의 셧다운까지 남은 시간(분)
    int inadvanceTimeInMinutes;

    /// 셧다운 알림 메세지
    wchar_t* message;

    /// 메세지 노출 시간(초)
    int exposureTimeInSeconds;
};
#pragma pack(pop)

/*!
*  @struct StovePCTraceHint
*  @brief StovePCTraceHint 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCTraceHint
{
    /// API 호출 결과 
    StovePCResult result;

    /// PCSDK가 초기화 될 때마다 발급되는 세션 아이디
    char* sessionId;

    /// 레퍼런스(런처 또는 SGA)가 실행될 때마다 발급되는 세션 아이디 
    char* refSessionId;

    /// 웹브라우저를 구분하기 위해 발급되는 아이디
    char* uuid;

    /// 레퍼런스(런처 또는 SGA)의 최초 구동 프로토콜
    char* serviceProtocol;

    /// 레퍼런스(런처 또는 SGA)의 원천 유형
    char* refSourceType;
};
#pragma pack(pop)

/*!
*  @struct StovePCShopCategory
*  @brief StovePCShopCategory 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCShopCategory
{
    /// 카테고리 아이디
    char* categoryId;

    /// 부모 카테고리 아이디
    char* parentCategoryId;

    /// 카테고리 순서
    int displayNo;

    /// 카테고리명
    wchar_t* name;

    /// 계층구조상의 깊이
    int depth;
};
#pragma pack(pop)

/*!
*  @struct StovePCProduct
*  @brief StovePCProduct 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCProduct
{
    /// 상품 아이디
    int64_t productId;

    /// 상품 아이디에 매핑되는 게임 내 아이템 아이디
    char* gameItemId;

    /// 상품명
    wchar_t* name;

    /// 상품 상세
    wchar_t* description;

    /// 개별 상품 수량
    int quantity;

    /// 아이템 유형 코드 (1:인디 패키지 게임 아이템, 2:인 게임 아이템, 3:패키지 아이템)
    short productTypeCode;

    /// 카테고리 아이디
    char* categoryId;

    /// 통화 코드
    char* currencyCode;

    /// 상품 정가(결제)
    double price;

    /// 상품 정가(전시)
    double displayPrice;

    /// 상품 정가 문자열(전시)
    char* displayPriceString;

    /// 상품 판매가(결제)
    double salePrice;

    /// 상품 판매가(전시)
    double displaySalePrice;

    /// 상품 판매가 문자열(전시)
    char* displaySalePriceString;

    /// 할인 여부
    bool isDiscount;

    /// 할인 유형 (1:정률, 2:정액)
    short discountType;

    /// 할인 값
    int discountTypeValue;

    /// 할인 시작 일자(epoch time milliseconds)
    uint64_t discountBeginDate;

    /// 할인 종료 일자(epoch time milliseconds)
    uint64_t discountEndDate;

    /// 상품 총 판매수량
    int totalQuantity;

    /// 회원 구매 수량
    int memberQuantity;

    /// guid 구매 수량
    int guidQuantity;

    /// 대표 상품 이미지
    char* thumbnailUrl;

    /// 청약 철회 가능 여부
    bool canWithdraw;
};
#pragma pack(pop)

/*!
*  @struct StovePCOrderProduct
*  @brief StovePCOrderProduct 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCOrderProduct
{
    /// 상품 아이디
    int64_t productId;

    /// 상품 판매가
    double salePrice;

    /// 수량
    int quantity;
};
#pragma pack(pop)

/*!
*  @struct StovePCPurchase
*  @brief StovePCPurchase 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCPurchase
{
    /// 거래 고유 마스터 번호
    int64_t transactionMasterNo;

    /// 1회용 결제 URL
    char* tempPaymentUrl;

    /// 구매 진행 상태
    int purchaseProgress;
};
#pragma pack(pop)

/*!
*  @struct StovePCPurchaseProduct
*  @brief StovePCPurchaseProduct 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCPurchaseProduct
{
    /// 상품 아이디
    int64_t productId;

    /// 카테고리 아이디
    char* categoryId;

    /// 상품 총 판매수량
    int totalQuantity;

    /// 회원 구매 수량
    int memberQuantity;

    /// guid 구매 수량
    int guidQuantity;
};
#pragma pack(pop)

/*!
*  @struct StovePCPurchaseVerification
*  @brief StovePCPurchaseVerification 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCPurchaseVerification
{
    /// API 호출 결과
    StovePCResult result;

    /// 구매여부 (true: 구매이력이 있음, false : 구매이력 없음)
    bool isPurchased;
};
#pragma pack(pop)

/*!
*  @struct StovePCInventoryItem
*  @brief StovePCInventoryItem 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCInventoryItem
{
    /// 거래 고유 마스터 번호
    int64_t transactionMasterNo;

    /// 거래 고유 상세 번호
    int64_t transactionDetailNo;

    /// 상품 아이디
    int64_t productId;

    /// 상품 아이디에 매핑되는 게임 내 아이템 아이디
    char* gameItemId;

    /// 상품명
    wchar_t* productName;

    /// 수량
    int quantity;

    /// 대표 상품 이미지
    char* thumbnailUrl;
};
#pragma pack(pop)

/*!
*  @struct StovePCTermsAgreement
*  @brief StovePCTermsAgreement 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCTermsAgreement
{
    /// 게임 아이디 
    wchar_t* gameId;

    /// 필수약관 전체 동의 여부
    bool result;

    /// 권역 코드
    char* region;

    /// 약관동의 URL
    char* agreementUrl;
};
#pragma pack(pop)

#endif //STOVEPCDEFINE_H_
